/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */

/*
 *  Caja
 *
 *  Copyright (C) 2004 Red Hat, Inc.
 *
 *  Caja is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  Caja is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public
 *  License along with this program; if not, write to the Free
 *  Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 *  Authors: Alexander Larsson <alexl@redhat.com>
 *
 */

#ifndef CAJA_ACTIONS_H
#define CAJA_ACTIONS_H

#define CAJA_ACTION_STOP "Stop"
#define CAJA_ACTION_RELOAD "Reload"
#define CAJA_ACTION_BACK "Back"
#define CAJA_ACTION_UP "Up"
#define CAJA_ACTION_UP_ACCEL "UpAccel"
#define CAJA_ACTION_FORWARD "Forward"
#define CAJA_ACTION_SHOW_HIDE_TOOLBAR "Show Hide Toolbar"
#define CAJA_ACTION_SHOW_HIDE_SIDEBAR "Show Hide Sidebar"
#define CAJA_ACTION_SHOW_HIDE_STATUSBAR "Show Hide Statusbar"
#define CAJA_ACTION_SHOW_HIDE_LOCATION_BAR "Show Hide Location Bar"
#define CAJA_ACTION_SHOW_HIDE_EXTRA_PANE "Show Hide Extra Pane"
#define CAJA_ACTION_GO_TO_BURN_CD "Go to Burn CD"
#define CAJA_ACTION_GO_TO_LOCATION "Go to Location"
#define CAJA_ACTION_GO_HOME "Home"
#define CAJA_ACTION_ADD_BOOKMARK "Add Bookmark"
#define CAJA_ACTION_EDIT_BOOKMARKS "Edit Bookmarks"
#define CAJA_ACTION_HOME "Home"
#define CAJA_ACTION_ZOOM_IN "Zoom In"
#define CAJA_ACTION_ZOOM_OUT "Zoom Out"
#define CAJA_ACTION_ZOOM_NORMAL "Zoom Normal"
#define CAJA_ACTION_SHOW_HIDDEN_FILES "Show Hidden Files"
#define CAJA_ACTION_CLOSE "Close"
#define CAJA_ACTION_SEARCH "Search"
#define CAJA_ACTION_FOLDER_WINDOW "Folder Window"
#define CAJA_ACTION_NEW_TAB "New Tab"

#endif /* CAJA_ACTIONS_H */
